@@ -33,32 +33,32 @@
  *  POSSIBILITY OF SUCH DAMAGE.
  */ 
 
-/** \author Jia Pan */
+/** @author Jia Pan */
 
 #ifndef FCL_BROAD_PHASE_BRUTE_FORCE_H
 #define FCL_BROAD_PHASE_BRUTE_FORCE_H
 
-#include "fcl/broadphase/broadphase.h"
 #include <list>
-
+#include "fcl/broadphase/broadphase_collision_manager.h"
 
 namespace fcl
 {
 
 /// @brief Brute force N-body collision manager
-class NaiveCollisionManager : public BroadPhaseCollisionManager
+template <typename S>
+class FCL_EXPORT NaiveCollisionManager : public BroadPhaseCollisionManager<S>
 {
 public:
-  NaiveCollisionManager() {}
+  NaiveCollisionManager();
 
   /// @brief add objects to the manager
-  void registerObjects(const std::vector<CollisionObject*>& other_objs);
+  void registerObjects(const std::vector<CollisionObject<S>*>& other_objs);
 
   /// @brief add one object to the manager
-  void registerObject(CollisionObject* obj);
+  void registerObject(CollisionObject<S>* obj);
 
   /// @brief remove one object from the manager
-  void unregisterObject(CollisionObject* obj);
+  void unregisterObject(CollisionObject<S>* obj);
 
   /// @brief initialize the manager, related with the specific type of manager
   void setup();
@@ -70,39 +70,43 @@ public:
   void clear();
 
   /// @brief return the objects managed by the manager
-  void getObjects(std::vector<CollisionObject*>& objs) const;
+  void getObjects(std::vector<CollisionObject<S>*>& objs) const;
 
   /// @brief perform collision test between one object and all the objects belonging to the manager
-  void collide(CollisionObject* obj, void* cdata, CollisionCallBack callback) const;
+  void collide(CollisionObject<S>* obj, void* cdata, CollisionCallBack<S> callback) const;
 
   /// @brief perform distance computation between one object and all the objects belonging to the manager
-  void distance(CollisionObject* obj, void* cdata, DistanceCallBack callback) const;
+  void distance(CollisionObject<S>* obj, void* cdata, DistanceCallBack<S> callback) const;
 
   /// @brief perform collision test for the objects belonging to the manager (i.e., N^2 self collision)
-  void collide(void* cdata, CollisionCallBack callback) const;
+  void collide(void* cdata, CollisionCallBack<S> callback) const;
 
   /// @brief perform distance test for the objects belonging to the manager (i.e., N^2 self distance)
-  void distance(void* cdata, DistanceCallBack callback) const;
+  void distance(void* cdata, DistanceCallBack<S> callback) const;
 
   /// @brief perform collision test with objects belonging to another manager
-  void collide(BroadPhaseCollisionManager* other_manager, void* cdata, CollisionCallBack callback) const;
+  void collide(BroadPhaseCollisionManager<S>* other_manager, void* cdata, CollisionCallBack<S> callback) const;
 
   /// @brief perform distance test with objects belonging to another manager
-  void distance(BroadPhaseCollisionManager* other_manager, void* cdata, DistanceCallBack callback) const;
+  void distance(BroadPhaseCollisionManager<S>* other_manager, void* cdata, DistanceCallBack<S> callback) const;
 
   /// @brief whether the manager is empty
   bool empty() const;
   
   /// @brief the number of objects managed by the manager
-  inline size_t size() const { return objs.size(); }
+  size_t size() const;
 
 protected:
 
   /// @brief objects belonging to the manager are stored in a list structure
-  std::list<CollisionObject*> objs;
+  std::list<CollisionObject<S>*> objs;
 };
 
+using NaiveCollisionManagerf = NaiveCollisionManager<float>;
+using NaiveCollisionManagerd = NaiveCollisionManager<double>;
 
-}
+} // namespace fcl
+
+#include "fcl/broadphase/broadphase_bruteforce-inl.h"
 
 #endif
